﻿using System.Collections.ObjectModel;
using AsReaderP252BiOSDemo.Platforms.iOS;
using AsReaderP252BiOSSDK;
using Foundation;

namespace AsReaderP252BiOSDemo;

public partial class BarcodeScanPage : ContentPage
{

    private AsReaderP252BMAUIManager asReaderP252BMAUIManager;
    private AsReaderP252BSDelegate asReaderP252BDelegate;
    private ObservableCollection<ItemData> BarcodeItems = new ObservableCollection<ItemData>();
    public ObservableCollection<ItemData> ItemData { get { return BarcodeItems; } }
    private Dictionary<string, ItemData> dataDic = new Dictionary<string, ItemData>();
    private AsReaderP252BCommandType actionState;
    private NSStringEncoding charset;
    int count;

    public BarcodeScanPage()
	{
        InitializeComponent();
        count = 0;
        charset = NSStringEncoding.ASCIIStringEncoding;
        actionState = AsReaderP252BCommandType.Stop;
        asReaderP252BMAUIManager = AsReaderP252BMAUIManager.sharedInstance();
        asReaderP252BDelegate = AsReaderP252BSDelegate.sharedInstance();
    }
    protected override void OnAppearing()
    {
        base.OnAppearing();
        BarcodeListView.ItemsSource = BarcodeItems;
        asReaderP252BMAUIManager.SetScanMode(AsReaderP252BScanMode.BarcodeScanMode);
        asReaderP252BDelegate.WhenAsReaderP252BIsConnected = (bool isConnected) =>
        {
            WhenAsReaderP252BIsConnected(isConnected);
        };
        asReaderP252BDelegate.OnAsReaderP252BTriggerKeyEvent = (bool status) =>
        {
            bool sta = OnAsReaderP252BTriggerKeyEvent(status);
            return sta;
        };
        asReaderP252BDelegate.DetectBarcode = (string barcodeType, NSData barcodeData) =>
        {
            NSString barcode = new NSString();
            try
            {
                barcode = new NSString(barcodeData, charset);
            }
            catch
            {
                try
                {
                    barcode = new NSString(barcodeData, NSStringEncoding.ASCIIStringEncoding);
                }
                catch
                {
                    return;
                }
            }
            if (barcode == null)
            {
                barcode = new NSString(barcodeData, NSStringEncoding.ASCIIStringEncoding);
            }
            DetectBarcode(barcodeType, barcode);
        };
        asReaderP252BDelegate.ChangedAsReaderP252BActionState = (AsReaderP252BCommandType action, nint resultCode) =>
        {
            ChangedAsReaderP252BActionState(action, resultCode);
        }; 
    }
    protected override void OnDisappearing()
    {
        base.OnDisappearing();
        asReaderP252BMAUIManager.StopDecode();
    }
    void StartScanButton_Clicked(object sender, EventArgs e)
    {
        if (actionState == AsReaderP252BCommandType.DecodeStart)
        {
            asReaderP252BMAUIManager.StopDecode();
        }
        else if (actionState == AsReaderP252BCommandType.Stop)
        {
            asReaderP252BMAUIManager.StartDecode();
        }
    }
    void ClearScanButton_Clicked(object sender, EventArgs e)
    {
        count = 0;
        CountLabel.Text = count.ToString();
        dataDic.Clear();
        BarcodeItems.Clear();
    }
    void ASCIIButton_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Black;
        UTF8Button.TextColor = Colors.Gray;
        ShiftJISButton.TextColor = Colors.Gray;
        charset = NSStringEncoding.ASCIIStringEncoding;
    }
    void UTF8Button_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Gray;
        UTF8Button.TextColor = Colors.Black;
        ShiftJISButton.TextColor = Colors.Gray;
        charset = NSStringEncoding.UTF8;
    }
    void ShiftJISButton_Clicked(object sender, EventArgs e)
    {
        ASCIIButton.TextColor = Colors.Gray;
        UTF8Button.TextColor = Colors.Gray;
        ShiftJISButton.TextColor = Colors.Black;
        charset = NSStringEncoding.ShiftJIS;
    }
    private void WhenAsReaderP252BIsConnected(bool isConnected)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (!isConnected)
            {
                Navigation.PopAsync();
            }
        });
    }
    private void DetectBarcode(string barcodeType, string barcodeData)
    {
        ItemData InventoryDataChanged;
        if (dataDic.ContainsKey(barcodeType + barcodeData))
        {
            InventoryDataChanged = dataDic[barcodeType + barcodeData];
            InventoryDataChanged.Count++;
            dataDic[barcodeType + barcodeData] = InventoryDataChanged;
        }
        else
        {
            InventoryDataChanged = new ItemData(barcodeType, barcodeData, 1);
            dataDic.Add(barcodeType + barcodeData, InventoryDataChanged);
            BarcodeItems.Add(InventoryDataChanged);
        }
        count++;
        MainThread.BeginInvokeOnMainThread(() =>
        {
            BarcodeListView.ItemsSource = null;
            BarcodeListView.ItemsSource = BarcodeItems;
            CountLabel.Text = count.ToString();
        });
    }
    private bool OnAsReaderP252BTriggerKeyEvent(bool status)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (status)
            {
                asReaderP252BMAUIManager.StartDecode();
            }
            else
            {
                asReaderP252BMAUIManager.StopDecode();
            }
        });
        return false;
    }
    private void ChangedAsReaderP252BActionState(AsReaderP252BCommandType action, nint resultCode)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            actionState = action;
            if (action == AsReaderP252BCommandType.DecodeStart)
            {
                StartScanButton.Text = "Stop Scan";
            }
            else if (action == AsReaderP252BCommandType.Stop)
            {
                StartScanButton.Text = "Start Scan";
            }
        });
    }
}
